/*
 * CommentUtils.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1;

import java.awt.Point;
import org.netbeans.modules.schema2beans.BaseBean;
import org.w3c.dom.Comment;

/**
 *
 * @author markus
 */
public class CommentUtils {

    private static final String POS_TOKEN = "nbm-pos,";
    private static final String ZOOM_TOKEN = "nbm-zoom,";

    public static void saveLocation(BaseBean component, Point location) {
        String newData = POS_TOKEN + location.x + "," + location.y;

        Comment oldComment = findComment(POS_TOKEN, component.comments());
        if (oldComment != null) {
            oldComment.setData(newData);
        } else {
            component.addComment(newData);
        }
    }

    public static Point getLocation(BaseBean component) {
        Point point = null;
        Comment comment = findComment(POS_TOKEN, component.comments());

        if (comment != null) {
            String[] data = comment.getTextContent().split(",");
            if (data.length >= 2) {
                try {
                    point = new Point(Integer.parseInt(data[1]), Integer.parseInt(data[2]));
                } catch (NumberFormatException ex) {
                    System.err.println("Bad format on location-comment: \"" + comment.getTextContent() + "\"");
                }
            }
        }
        return point;
    }

    public static Double getZoomFactor(BaseBean component) {
        Double zoom = null;
        Comment comment = findComment(ZOOM_TOKEN, component.comments());

        if (comment != null) {
            String[] data = comment.getTextContent().split(",");
            if (data.length >= 1) {
                try {
                    zoom = Double.parseDouble(data[1]);
                } catch (NumberFormatException ex) {
                    System.err.println("Bad format on zoom-comment: \"" + comment.getTextContent() + "\"");
                }
            }
        }
        return zoom;
    }

    public static void saveZoomFactor(BaseBean component, double zoom) {
        String newData = ZOOM_TOKEN + zoom;

        Comment oldComment = findComment(ZOOM_TOKEN, component.comments());
        if (oldComment != null) {
            oldComment.setData(newData);
        } else {
            component.addComment(newData);
        }
    }

    private static Comment findComment(String contains, Comment[] comments) {
        for (Comment c : comments) {
            if (c.getData().contains(contains)) {
                return c;
            }
        }
        return null;
    }
}
