/*
 * ADLDataObject.java
 * 
 * 
 * Copyright (C) 2008 Markus Kilås
 *  
 * This file is part of TBN NetBeans Module.
 *
 * TBN NetBeans Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TBN NetBeans Module is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TBN NetBeans Module.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package com.markuspage.markus.tbn.module1.adl;

import com.markuspage.markus.tbn.module1.TopMultiviewElement;
import com.markuspage.markus.tbn.module1.adl.model.Configuration;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;
import org.netbeans.api.xml.cookies.CheckXMLCookie;
import org.netbeans.api.xml.cookies.ValidateXMLCookie;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.modules.schema2beans.Schema2BeansException;
import org.netbeans.modules.xml.multiview.DesignMultiViewDesc;
import org.netbeans.modules.xml.multiview.XmlMultiViewDataObject;
import org.netbeans.modules.xml.multiview.XmlMultiViewDataSynchronizer;
import org.netbeans.spi.xml.cookies.CheckXMLSupport;
import org.netbeans.spi.xml.cookies.DataObjectAdapters;
import org.netbeans.spi.xml.cookies.ValidateXMLSupport;
import org.openide.ErrorManager;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.nodes.CookieSet;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.text.DataEditorSupport;

public class ADLDataObject extends XmlMultiViewDataObject {

    private static final String ADL_ICON = "com/markuspage/markus/tbn/module1/adl/adl16.png";
    private ModelSynchronizer modelSynchronizer;
    private Configuration book;

    public ADLDataObject(FileObject pf, ADLDataLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        CookieSet cookies = getCookieSet();
        cookies.add((Node.Cookie) DataEditorSupport.create(this, getPrimaryEntry(), cookies));

        modelSynchronizer = new ModelSynchronizer(this);
        org.xml.sax.InputSource in = DataObjectAdapters.inputSource(this);
        CheckXMLCookie checkCookie = new CheckXMLSupport(in);
        getCookieSet().add(checkCookie);
        ValidateXMLCookie validateCookie = new ValidateXMLSupport(in);
        getCookieSet().add(validateCookie);
        try {
            parseDocument();
        } catch (IOException ex) {
            System.out.println("ex=" + ex);
        }
    }

    @Override
    protected Node createNodeDelegate() {
        return new ADLDataNode(this, getLookup());
    }

    @Override
    public Lookup getLookup() {
        return getCookieSet().getLookup();
    }

    @Override
    protected DesignMultiViewDesc[] getMultiViewDesc() {
        return new DesignMultiViewDesc[] {
            new DesignView(this),
            new GraphView(this)
        };
    }

    @Override
    protected String getPrefixMark() {
        return null;
    }

    public void modelUpdatedFromUI() {
        modelSynchronizer.requestUpdateData();
    }

    private void parseDocument() throws IOException {
        if(book == null) {
            book = getConfiguration();
        } else {
            java.io.InputStream is = getEditorSupport().getInputStream();
            Configuration newConfiguration = null;
            try {
                newConfiguration = Configuration.createGraph(is);
            } catch (RuntimeException ex) {
                System.out.println("runtime error " + ex);
            }
            if (newConfiguration != null) {
                book.merge(newConfiguration, org.netbeans.modules.schema2beans.BaseBean.MERGE_UPDATE);
            }
        }
    }

    public Configuration getConfiguration() throws IOException {
        System.out.println("Primary: " + getPrimaryFile());

        if(book == null) {
            book = Configuration.createGraph(getPrimaryFile().getInputStream());
        } //FileUtil.toFile(getPrimaryFile()));
        return book;
    }

    private class ModelSynchronizer extends XmlMultiViewDataSynchronizer {

        public ModelSynchronizer(XmlMultiViewDataObject dataObject) {
            super(dataObject, 500);
        }

        protected boolean mayUpdateData(boolean allowDialog) {
            return true;
        }

        protected void updateDataFromModel(Object model, org.openide.filesystems.FileLock lock, boolean modify) {
            try {
                if(model == null) {
                    return;
                }

                Writer out = new StringWriter();
                ((Configuration) model).write(out);
                out.close();
                getDataCache().setData(lock, out.toString(), modify);
            } catch (IOException ex) {
                Exceptions.printStackTrace(ex);
            } catch (Schema2BeansException ex) {
                Exceptions.printStackTrace(ex);
            }
        }

        protected Object getModel() {
            try {
                return getConfiguration();
            } catch (IOException ex) {
                ErrorManager.getDefault().notify(org.openide.ErrorManager.INFORMATIONAL, ex);
                return null;
            }
        }

        protected void reloadModelFromData() {
            try {
                parseDocument();
            } catch (IOException e) {
                ErrorManager.getDefault().notify(ErrorManager.INFORMATIONAL, e);
            }
        }
    }

    private static class DesignView extends DesignMultiViewDesc {


        DesignView(ADLDataObject dObj) {
            super(dObj, "Design");
        }

        public MultiViewElement createElement() {
            ADLDataObject dObj = (ADLDataObject) getDataObject();
            return new ADLToolBarMVElement(dObj);
        }

        public java.awt.Image getIcon() {
            return org.openide.util.Utilities.loadImage(ADL_ICON); //NOI18N
        }

        public String preferredID() {
            return "adl_multiview_design";
        }
    }

    private static class GraphView extends DesignMultiViewDesc {

        GraphView(ADLDataObject dObj) {
            super(dObj, "Graph");
        }

        public org.netbeans.core.spi.multiview.MultiViewElement createElement() {
            ADLDataObject dObj = (ADLDataObject) getDataObject();
            return new TopMultiviewElement(dObj, new ADLGraphPanel(dObj));
        }

        public java.awt.Image getIcon() {
            return org.openide.util.Utilities.loadImage(ADL_ICON); //NOI18N
        }

        public String preferredID() {
            return "adl_multiview_graph";
        }
    }
}
